package org.rcisoft.code.controller;

import io.swagger.annotations.ApiOperation;
import org.rcisoft.code.model.CodeSchema;
import org.rcisoft.code.model.CodeTable;
import org.rcisoft.code.service.CodeService;
import org.rcisoft.common.controller.PaginationController;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.ServletOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;

/**
 * Created by lcy on 17/6/28.
 */
@RestController
@RequestMapping("/code")
public class CodeController extends PaginationController<CodeTable> {


    @Autowired
    private CodeService codeServiceImpl;

    /**
     * 连接数据库    schema
     * @param codeSchema
     * @return
     */
    @ApiOperation(value="构建  treeView", notes="构建  treeView")
    @RequestMapping("connectDb")
    public Result connectDb(CodeSchema codeSchema){
        List<CodeTable> result = codeServiceImpl.getCodeTablesBySchema(codeSchema);
        if(null == result || 0 == result.size())
            return Result.builder(new PersistModel(0),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    "数据库连接信息错误,或数据库不存在数据表!",
                    result);
        else
            return Result.builder(new PersistModel(1),
                    MessageConstant.MESSAGE_ALERT_SUCCESS,
                    "数据库连接信息错误,或数据库不存在数据表!",
                    result);
    }

    /**
     * 导出code package
     * @return
     */
    @ApiOperation(value="构建  treeView", notes="构建  treeView")
    @RequestMapping(value = "constructCode")
    //@ResponseBody   // springMVC 下载 报页面找不到
    public void constructCode(CodeSchema codeSchema) throws IOException{
        ServletOutputStream out = null;
        InputStream fin = null;
        File file = null;
        //codeSchema.setUserId(UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID));
        String path = codeServiceImpl.generateCode(codeSchema);
        try{
            if(null == path)
                return ;
            file = new File(path);
            //不存在
            if(null == file || !file.exists())
                return ;
            // response
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/force-download");
            response.addHeader("Content-Disposition", "attachment;filename=cyCode.zip");
            out = response.getOutputStream();
            fin = new FileInputStream(file);
            byte[] buffer = new byte[1024];//缓冲区
            int bytesToRead = -1;
            // 通过循环将读入的Word文件的内容输出到浏览器中
            while((bytesToRead = fin.read(buffer)) != -1) {
                out.write(buffer, 0, bytesToRead);
            }
        }catch(Exception ex){
            ex.printStackTrace();
        }finally{
            if(fin != null) fin.close();
            if(out != null) out.close();
            //if(file != null) file.delete(); // 删除临时文件
        }
    }

    /**
     * 跳转 freemarker 连接
     * @return
     */
    @GetMapping("connect")
    public ModelAndView connectPage(){
        ModelAndView mv = new ModelAndView("connect");
        return mv;
    }


}
