package org.rcisoft.core.controller;

import org.rcisoft.core.result.Result;
import org.rcisoft.core.service.AuthenticationService;
import org.rcisoft.core.util.ResultGenerator;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * Created by lcy on 17/11/24.
 */
@RestController
@RequestMapping("/auth")
public class AuthenticationController {

    @Value("${jwt.header}")
    private String tokenHeader;

    @Autowired
    private AuthenticationService authenticationServiceImpl;

    /**
     * 登录
     * @param username
     * @param password
     * @return
     */
    @PostMapping(value = "${jwt.route.authentication.path}")
    public Result login(@RequestParam("username")String username,
                        @RequestParam("password")String password,
                        @RequestParam("userType")String userType){
        final String token = authenticationServiceImpl.login(username, password, userType);
        Result result = ResultGenerator.genSuccessResult(token);
        return result;
    }

    /**
     * 注册
     * @param addedUser
     * @return
     */
    @PostMapping(value = "${jwt.route.authentication.register}")
    public Result register(SysUser addedUser){
        authenticationServiceImpl.register(addedUser);
        return ResultGenerator.genSuccessResult();
    }

    @GetMapping(value = "${jwt.route.authentication.refresh}")
    public ResponseEntity<?> refreshAndGetAuthenticationToken(
            HttpServletRequest request){
        String token = request.getHeader(tokenHeader);
        String refreshedToken = authenticationServiceImpl.refresh(token);
        if(refreshedToken == null) {
            return ResponseEntity.badRequest().body(null);
        } else {
            return ResponseEntity.ok(refreshedToken);
        }
    }


}
