package org.rcisoft.core.util;

import io.jsonwebtoken.Claims;
import org.apache.commons.collections.map.HashedMap;
import org.rcisoft.core.bean.JwtBean;
import org.rcisoft.core.model.JwtUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Created by lcy on 17/11/21. *
 * header.payload.signature
 *
 *
 */
@Component
public class JwtUtil {

    public JwtUtil() {
    }

    private static JwtBean jwtBean;

    @Autowired
    private JwtUtil(JwtBean jwtBean){
        this.jwtBean = jwtBean;
    }


    /**
     * 根据token获取用户名
     */
    public static String getUsernameFromToken(String token){
        try {
            Claims claims = jwtBean.getClaimsFromToken(token);
            String username = claims.getSubject();
            return username;
        }catch (Exception e){
            return null;
        }
    }

    /**
     * 设置用户信息进jwt
     */
    public static String generateToken(UserDetails userDetails){
        JwtUser jwtUser = (JwtUser)userDetails;
        Map<String,Object> map = new HashedMap();
        map.put(userDetails.getUsername(),userDetails);
        map.put(UserUtil.USER_ID,jwtUser.getBusinessId());
        map.put(UserUtil.USER_USERNAME,jwtUser.getUsername());
        String token = jwtBean.generateToken(userDetails.getUsername(),map);
        return token;
    }

    /**
     * 验证token是否正确
     * @param token
     * @param userDetails
     * @return
     */
    public static boolean validateToken(String token,UserDetails userDetails){
        Claims claims = jwtBean.getClaimsFromToken(token);
        LinkedHashMap<String,Object> tokenUserDetails = (LinkedHashMap<String,Object>) claims.get(userDetails.getUsername());
        boolean usernameValid = userDetails.getUsername().equals(tokenUserDetails.get("username"));
        boolean passwordValid = userDetails.getPassword().equals(tokenUserDetails.get("password"));
        if(usernameValid||passwordValid) // ?? &&
            return true;
        else
            return false;
    }
    /*
        获取 user claims
     */
    public static Claims getUserClaimsFromToken(String token){
        try {
            Claims claims = jwtBean.getClaimsFromToken(token);
            return claims;
        }catch (Exception e){
            return null;
        }
    }
}
