package org.rcisoft.sys.role.dao;

import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;
import org.rcisoft.core.base.BaseMapper;
import org.rcisoft.sys.dept.entity.DeptRole;
import org.rcisoft.sys.role.entity.SysRole;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * Created with family.
 * author: cy
 * Date: 16/6/2
 * Time: 上午10:03
 * description:
 */
@Repository
public interface SysRoleRepository extends BaseMapper<SysRole> {
    /**
     * 分页查询 roles
     * @param sysRole
     * @return
     */
    @Select("<script>select sys_role.*,sys_user.NAME as username from sys_role  LEFT JOIN sys_user ON sys_role.CREATE_BY=sys_user.BUSINESS_ID where "
    		+ "sys_role.DEL_FLAG='0' <if test=\"name !=null \"> and sys_role.NAME like concat('%',#{name},'%') </if> "
    		+ " <if test=\"flag !=null \">and sys_role.FLAG like concat('%',#{flag},'%') </if> "
    		//+ "<if test=\"phone !=null \">phone = #{phone} </if> <if test=\"usermail !=null \">usermail = #{usermail } </if>"
    		+ "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<SysRole> queryRoles(SysRole sysRole);
    
    
    @Select("<script>select * from sys_role  where "
    		+ "DEL_FLAG='0' "
    		+ "</script>")
    @ResultMap(value = "BaseResultMap" )
    List<SysRole> queryRolesSelect(SysRole sysRole);

    /**
     * 根据角色id  删除对应权限
     * @param roleId
     * @return
     */
    @Delete("<script>delete from sys_role_menu where role_id = #{roleId}</script>")
    int deleteRoleMenus(@Param("roleId") String roleId);
    
    
    /**
     * 根据角色id  删除对应部门
     * @param roleId
     * @return
     */
    @Delete("<script>delete from sys_dept_role where role_id = #{roleId}</script>")
    int deleteRoleDepts(@Param("roleId") String roleId);

    /**
     * 批量插入         角色_权限
     * @param menuRole
     * @return
     */
    int insertBatchRoleMenu(List<Map> menuRole);
    
    
    /**
     * 批量插入         角色_部门
     * @param       deptRole
     * @return
     */
    int insertBatchRoleDept(List<Map> deptRole);
    /**
     * 分页查询 roles
     * @param sysRole
     * @return
     */
    @Select(" <script>select "
    		+ " dr.BUSINESS_ID drid,r.BUSINESS_ID rid,r.name rname,r.code rcode,d.BUSINESS_ID did,d.name dname,d.dept_code deptcode, "
    		+ " IF(( "
    		+ " SELECT count(*) from sys_dept_role_user dru where dru.USER_ID=#{userId} "
    		+ " and dru.DEPT_ROLE_ID=dr.BUSINESS_ID"
    		+ " )>0,'1','0') as checked "
    		+ " FROM  sys_dept_role dr "
    		+ " LEFT JOIN sys_dept d ON d.BUSINESS_ID = dr.DEPT_ID "
    		+ " LEFT JOIN sys_role r on dr.ROLE_ID = r.BUSINESS_ID</script>")
    @ResultMap(value = "drResultMap" )
    List<DeptRole> queryUserRolesByPagination(@Param("userId") String userId, SysRole sysRole) ;
    
    
    /**
     * 分页查询 roles
     * @param sysRole
     * @return
     */
    @Select("<script>SELECT " +
            " sysRole.BUSINESS_ID, " +
            " sysRole.NAME, " +
            " sysRole.CODE, " +
            " sysRole.DATA_SCOPE, " +
            " sysRole.FLAG, " +
            " sysRole.DEL_FLAG, " +
            " sysRole.CREATE_BY, " +
            " sysRole.create_date, " +
            "IF ( " +
            " ( " +
            "  SELECT " +
            "   count(*) " +
            "  FROM " +
            "   SYS_DEPT_ROLE dr " +
            "  WHERE " +
            "   dr.ROLE_ID = sysRole.BUSINESS_ID " +
            "  AND ur.USER_ID = #{deptId}  " +
            " ) > 0, " +
            "  '1', " +
            "  '0' " +
            " ) AS checked " +
            "FROM " +
            " sys_role sysRole " +
            "WHERE " +
            " DEL_FLAG = '0' and FLAG = '1'</script>")
    @ResultMap(value = "BaseResultMap" )
    List<SysRole> queryDeptRolesByPagination(@Param("deptId") String deptId, SysRole sysRole);
     
    @Delete("<script>DELETE from sys_dept_role_user where DEPT_ROLE_ID="
    		+ "("
    		+ "SELECT BUSINESS_ID from sys_dept_role  where ROLE_ID=#{roleId}"
    		+ ")</script>")
    int deleteDeptRoleUserByroleId(@Param("roleId") String roleId);

    /**
     * 删除岗位
      * @param roleId
     * @return
     */
    @Delete("<script>DELETE from sys_dept_role where ROLE_ID=#{roleId}</script>")
    int deleteDeptRoleByRoleId(@Param("roleId") String roleId);

    /**
     * 删除权限
     * @param roleId
     * @return
     */
    @Delete("<script>DELETE from sys_role_menu where ROLE_ID=#{roleId}</script>")
    int deleteRoleMenuByRoleId(@Param("roleId") String roleId);

    @Select("<script>SELECT sr.* from s_role sr " +
            "LEFT JOIN s_r_user_role sur ON sr.business_id = sur.role_id " +
            "LEFT JOIN s_user su on su.business_id = sur.user_id " +
            "where su.business_id = #{businessId} " +
            "and sr.del_flag = '0' " +
            "and sr.flag = '1'</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysRole> findRolesByUserId(String businessId);

    @Select("<script>SELECT sr.* from s_role sr " +
            "LEFT JOIN s_r_user_role sur ON sr.business_id = sur.role_id " +
            "LEFT JOIN s_user su on su.business_id = sur.user_id " +
            "where su.login_name = #{username} " +
            "and sr.del_flag = '0' " +
            "and sr.flag = '1'</script>")
    @ResultMap(value = "BaseResultMap")
    List<SysRole> queryCodeByUsername(String username);
}
